import asyncio
from datetime import datetime, timedelta
from telegram import Bot
from telegram.ext import Application, CommandHandler, MessageHandler, filters
import aiohttp  # For fetching data from URLs

# Replace with your bot's API token
# Loop Gatekeeper bot
BOT_TOKEN = "7664853316:AAEP2dAb3GuWLib6eJB1vu8DIR65HqlIBRA"
# Loop Development
# BOT_TOKEN = "7309367551:AAGJNNPPQroL7yqw7U7eeUmsOXrKVQgzAw0" 

# URL to fetch the keyword lists
KEYWORD_LIST_URL = "https://loopaddon.uk/files/PJ/BOT_DATA/loop_bot_keywords.json"

# Dictionaries to store user data
new_users = {}
user_warnings = {}

# Keyword lists
check_list1 = []
check_list2 = []
check_list3 = []

async def old_is_user_admin(chat, user_id):
    """Check if the user is an admin by getting chat administrators."""
    administrators = await chat.get_administrators()
    return any(admin.user.id == user_id for admin in administrators)

async def is_user_admin(chat, user_id):
    """Check if the user is an admin, including anonymous admins."""
    administrators = await chat.get_administrators()
    return any(
        admin.user.id == user_id or admin.is_anonymous
        for admin in administrators
    )
   
async def fetch_keyword_lists():
    """Fetch keyword lists from a URL."""
    global check_list1, check_list2, check_list3
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(KEYWORD_LIST_URL) as response:
                if response.status == 200:
                    data = await response.json()
                    check_list1 = data.get("check_list1", [])
                    check_list2 = data.get("check_list2", [])
                    check_list3 = data.get("check_list3", [])
                    print("Keyword lists updated successfully.")
                else:
                    print(f"Failed to fetch keyword lists. HTTP Status: {response.status}")
    except Exception as e:
        print(f"Error fetching keyword lists: {e}")

async def periodic_keyword_update(interval=1800):
    """Periodically update the keyword lists."""
    while True:
        await fetch_keyword_lists()
        await asyncio.sleep(interval)  # Wait for the specified interval before updating again

async def start(update, context):
    """Command to check if the bot is running."""    
    user_id = update.effective_user.id
    chat = update.effective_chat
    
    # Check if the user is an admin
    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return
        
    await update.message.reply_text(
        f"Gate Keeper is active! \n\n"
        "Be aware there is a 3 strike warning process in place. \n\n"
        "Any user that receives 3 warnings will be automatically removed from the group."
    )

async def welcome_new_user(update, context):
    """Handles new users joining the group."""
    if update.message.new_chat_members:
        for user in update.message.new_chat_members:
            user_id = user.id
            chat_id = update.message.chat.id
            first_name = user.first_name
            
            bot_temp = Bot(BOT_TOKEN)
            try :
                user_photos = await bot_temp.get_user_profile_photos(user_id)
                pic_count = user_photos.total_count
            except :
                pic_count = 0

            if pic_count == 0:
                await context.bot.send_message(
                    chat_id=chat_id,
                    text=f"Welcome {first_name}! \n\nPlease ensure you have a profile picture as it's a requirement of this group. You will be automatically removed from this group if you do not have a profile picture within 24 hours of joining."
                )
            else : 
                await context.bot.send_message(
                    chat_id=chat_id,
                    text=f"Welcome {first_name}! \n\nPlease see pinned post for installation instructions."
                )

            new_users[user_id] = {"chat_id": chat_id, "join_time": datetime.now()}

async def warn_user(update, context):
    """Issues a manual warning to a user."""    
    user_id = update.effective_user.id
    chat = update.effective_chat
    
    # Check if the user is an admin
    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return
        
    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        user_id = warned_user.id
        chat_id = update.message.chat.id
        ####
        # Get the reason from the command argument
        # reason = " ".join(context.args) if context.args else "No reason provided."

        if user_id not in user_warnings:
            user_warnings[user_id] = 0

        user_warnings[user_id] += 1
        warnings = user_warnings[user_id]
        
        # await update.message.reply_text(
            # f"{warned_user.first_name} has been warned. \nReason: {reason}\n\nTotal warnings: {warnings}."
        # )

        await update.message.reply_text(
            f"{warned_user.first_name} has been warned. \nTotal warnings: {warnings}."
        )

        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, user_id)
                await context.bot.send_message(
                    chat_id, f"{warned_user.first_name} has been removed for receiving 3 warnings."
                )
                del user_warnings[user_id]
            except Exception as e:
                await update.message.reply_text(f"Failed to remove {warned_user.first_name}. Error: {e}")
    else:
        await update.message.reply_text("Please reply to a user's message to warn them.")

async def reset_warnings(update, context):
    """Resets warnings for a user."""
    user_id = update.effective_user.id
    chat = update.effective_chat
    
    # Check if the user is an admin
    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return
        
    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        user_id = warned_user.id

        if user_id in user_warnings:
            del user_warnings[user_id]
            await update.message.reply_text(f"Warnings for {warned_user.first_name} have been reset.")
        else:
            await update.message.reply_text(f"{warned_user.first_name} has no warnings.")
    else:
        await update.message.reply_text("Please reply to a user's message to reset their warnings.")

async def ban_user(update, context):
    """Manually bans a user with a reason."""
    user_id = update.effective_user.id
    chat = update.effective_chat
    
    # Check if the user is an admin
    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return
        
    if update.message.reply_to_message:
        banned_user = update.message.reply_to_message.from_user
        user_id = banned_user.id
        chat_id = update.message.chat.id

        # Get the reason from the command arguments
        reason = " ".join(context.args) if context.args else "No reason provided."

        try:
            # Ban the user
            await context.bot.ban_chat_member(chat_id, user_id)
            await context.bot.send_message(
                chat_id,
                f"{banned_user.first_name} has been banned. \nReason: {reason}"
            )
        except Exception as e:
            await update.message.reply_text(f"Failed to ban {banned_user.first_name}. Error: {e}")
    else:
        await update.message.reply_text("Please reply to a user's message to ban them.")

async def handle_message(update, context):
    if not update.message or not update.message.text:
        return

    text = update.message.text.lower()
    user_id = update.message.from_user.id
    first_name = update.message.from_user.first_name
    chat_id = update.message.chat.id

    async def issue_warning():        
        user_id = update.effective_user.id
        chat = update.effective_chat
    
        # Check if the user is an admin
        if not await is_user_admin(chat, user_id):
            await update.message.reply_text("You need to be an admin to use this command.")
            return
        
        if user_id not in user_warnings:
            user_warnings[user_id] = 0
        user_warnings[user_id] += 1
        warnings = user_warnings[user_id]

        await update.message.reply_text(
            f"Warning {first_name}! \nThis is warning {warnings}."
        )

        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, user_id)
                await context.bot.send_message(
                    chat_id, f"{first_name} has been removed for receiving 3 warnings."
                )
                del user_warnings[user_id]
            except Exception as e:
                await update.message.reply_text(f"Failed to remove {first_name}. Error: {e}")

    if any(check in text for check in check_list1):
        await update.message.reply_text(
            f"Warning {first_name}! \n\n"
            "The answer is always the same:\n\n"
            "If you've looked, you know the answer.\n\n"
            "If you've not looked, it's not there.\n\n"
            "If-Its-On-Its-In"
        )
        await issue_warning()

    elif any(check in text for check in check_list2):
        await update.message.reply_text(
            f"Warning {first_name}! \n\n"
            "Replays are added and announced as they become available."
        )
        await issue_warning()

    elif any(check in text for check in check_list3):
        await update.message.reply_text(
            f"{first_name} \n"
            "Please see pinned post for installation instructions."
        )

async def check_profiles():
    """Checks if new users have set profile pictures within 24 hours."""
    bot = Bot(BOT_TOKEN)
    now = datetime.now()

    for user_id, info in list(new_users.items()):
        chat_id = info["chat_id"]
        join_time = info["join_time"]

        # Check if 24 hours have passed
        if now - join_time >= timedelta(hours=24):
            try:
                # Fetch user's profile photos
                user_photos = await bot.get_user_profile_photos(user_id)

                if user_photos.total_count == 0:
                    # User has no profile picture, kick them
                    await bot.kick_chat_member(chat_id, user_id)
                    await bot.unban_chat_member(chat_id)  # Optional: Auto-unban after kick
                    await bot.send_message(chat_id, f"User was removed for not setting a profile picture.")
                
                # Remove user from tracking list
                del new_users[user_id]

            except Exception as e:
                print(f"Error checking or kicking user {user_id}: {e}")

async def profile_check_scheduler():
    """Asynchronously checks profiles every 30 minutes."""
    while True:
        await check_profiles()
        await asyncio.sleep(1800)  # Run every 30 minutes

async def run_bot():
    """Runs the bot and schedules tasks."""
    application = Application.builder().token(BOT_TOKEN).build()
    
    # Get Keywords from Server
    await fetch_keyword_lists()
    asyncio.create_task(periodic_keyword_update(1800))
    
    # Command handlers
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("warn", warn_user))
    application.add_handler(CommandHandler("reset", reset_warnings))
    application.add_handler(CommandHandler("ban", ban_user))  
    
    # Message handlers
    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS, welcome_new_user))
    application.add_handler(MessageHandler(filters.TEXT & ~filters.COMMAND, handle_message))

    # Start the profile check scheduler
    asyncio.create_task(profile_check_scheduler())
    
    # Manually handle the polling loop to prevent `run_polling()` from closing the event loop
    await application.initialize()
    await application.start()
    print("Bot is running...")
    try:
        await application.updater.start_polling()
        while True:
            try:
                await asyncio.sleep(1800)
            except KeyboardException:
                break
    finally:
        await application.stop()
        await application.shutdown()

def main():
    """Entry point for running the bot."""
    loop = asyncio.get_event_loop()
    loop.create_task(run_bot())
    loop.run_forever()

if __name__ == "__main__":
    main()
